/*
 * Created by IntelliJ IDEA.
 * User: plightbo
 * Date: Apr 29, 2002
 * Time: 10:41:55 PM
 */
package com.opensymphony.workflow;

import com.opensymphony.util.Logger;
import com.opensymphony.util.TextUtils;
import com.opensymphony.util.XMLUtils;
import com.opensymphony.util.EJBUtils;
import com.opensymphony.workflow.loader.*;
import com.opensymphony.workflow.persistence.WorkflowEntry;
import com.opensymphony.workflow.persistence.Persistence;
import com.opensymphony.workflow.persistence.PersistenceFactory;
import com.opensymphony.workflow.persistence.Step;
import com.opensymphony.workflow.config.ConfigLoader;
import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.module.propertyset.PropertyException;
import com.opensymphony.module.user.UserManager;
import com.opensymphony.module.user.User;
import com.opensymphony.module.user.EntityNotFoundException;

import java.util.*;
import java.io.InputStream;
import java.lang.reflect.Method;
import javax.ejb.EJBHome;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import bsh.Interpreter;
import bsh.EvalError;
import bsh.Primitive;
import bsh.NameSpace;

public class AbstractWorkflow implements Workflow {
    protected WorkflowContext context;
    private static Map workflows = new HashMap();
    private static final Logger log = new Logger(AbstractWorkflow.class);

    protected void loadConfig() {
        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        InputStream is = null;

        try {
            is = classLoader.getResourceAsStream("osworkflow.xml");
        } catch (Exception e) {
        }

        if (is == null) {
            try {
                is = classLoader.getResourceAsStream("/osworkflow.xml");
            } catch (Exception e) {
            }
        }

        if (is == null) {
            try {
                is = classLoader.getResourceAsStream("META-INF/osworkflow.xml");
            } catch (Exception e) {
            }
        }

        if (is == null) {
            try {
                is = classLoader.getResourceAsStream("/META-INF/osworkflow.xml");
            } catch (Exception e) {
            }
        }

        if(is!=null) ConfigLoader.load(is);
    }

    protected synchronized WorkflowDescriptor getWorkflow(String name) {
        WorkflowDescriptor wf = (WorkflowDescriptor) workflows.get(name);

        if (wf == null) {
            try {
                wf = ConfigLoader.getWorkflow(name);
            } catch (RuntimeException e) {
                throw new InternalWorkflowException(e);
            }
            workflows.put(name, wf);
        }
        return wf;
    }

    protected Map createVariableMap(WorkflowEntry entry, List registers) {
        Map map = new HashMap();
        PropertySet ps = entry.getPropertySet();

        // set entry variable
        map.put("entry", entry);

        // set context
        map.put("context", context);

        // prepare variables from PropertySet
        for (Iterator iterator = ps.getKeys().iterator(); iterator.hasNext();) {
            String key = (String) iterator.next();
            int type = ps.getType(key);
            switch (type) {
                case PropertySet.BOOLEAN:
                    map.put(key, new Boolean(ps.getBoolean(key)));
                    break;
                case PropertySet.DATA:
                    map.put(key, ps.getData(key));
                    break;
                case PropertySet.DATE:
                    map.put(key, ps.getDate(key));
                    break;
                case PropertySet.DOUBLE:
                    map.put(key, new Double(ps.getDouble(key)));
                    break;
                case PropertySet.INT:
                    map.put(key, new Integer(ps.getInt(key)));
                    break;
                case PropertySet.LONG:
                    map.put(key, new Long(ps.getLong(key)));
                    break;
                case PropertySet.OBJECT:
                    map.put(key, ps.getObject(key));
                    break;
                case PropertySet.PROPERTIES:
                    map.put(key, ps.getProperties(key));
                    break;
                case PropertySet.STRING:
                    map.put(key, ps.getString(key));
                    break;
                case PropertySet.TEXT:
                    map.put(key, ps.getText(key));
                    break;
                case PropertySet.XML:
                    map.put(key, ps.getXML(key));
                    break;
            }
        }

        // now talk to the registers for any extra objects needed in scope
        for (Iterator iterator = registers.iterator(); iterator.hasNext();) {
            RegisterDescriptor register = (RegisterDescriptor) iterator.next();
            if ("class".equals(register.getType())) {
                Register r = null;
                try {
                    r = (Register) Class.forName(register.getLocation()).newInstance();
                    map.put(register.getVariableName(), r.registerVariable(context, entry));
                } catch (Exception e) {
                    log.error("Could not utilize register class - skipping", e);
                }
            }
            else if ("remote-ejb".equals(register.getType())) {
                try {
                    EJBHome home = (EJBHome)PortableRemoteObject.narrow(new InitialContext().lookup(register.getLocation()), javax.ejb.EJBHome.class);
                    Method create = home.getClass().getMethod("create", new Class[0]);
                    RegisterRemote sessionBean = (RegisterRemote)create.invoke(home, new Object[0]);
                    map.put(register.getVariableName(), sessionBean.registerVariable(context, entry));
                } catch(Exception e) {
                    log.error("Could not create ejb register class at " + register.getLocation() + " - skipping", e);
                }
            }
            else if("local-ejb".equals(register.getType())) {
                try {
                    EJBHome home= (EJBHome)PortableRemoteObject.narrow(new InitialContext().lookup(register.getLocation()), javax.ejb.EJBHome.class);
                    Method create = home.getClass().getMethod("create", new Class[0]);
                    Register sessionBean = (Register)create.invoke(home, new Object[0]);
                    map.put(register.getVariableName(), sessionBean.registerVariable(context, entry));
                } catch(Exception e) {
                    log.error("Could not create local ejb register class at " + register.getLocation() + " - skipping", e);
                }
            }
            else if("jndi".equals(register.getType())) {
                try {
                    Register r = (Register)new InitialContext().lookup(register.getLocation());
                    map.put(register.getVariableName(), r.registerVariable(context, entry));
                } catch(Exception e) {
                    log.error("Could not create jndi register at " + register.getLocation() + " - skipping", e);
                }
            }
        }

        return map;
    }

    protected void setBshVariables(Interpreter i, Map variables) throws EvalError {
        Set entrySet = variables.entrySet();
        for (Iterator iterator = entrySet.iterator(); iterator.hasNext();) {
            Map.Entry entry = (Map.Entry) iterator.next();
            Object o = entry.getValue();
            String key = (String) entry.getKey();
            if (o instanceof Boolean) {
                i.set(key, ((Boolean) o).booleanValue());
            } else if (o instanceof Double) {
                i.set(key, ((Double) o).doubleValue());
            } else if (o instanceof Integer) {
                i.set(key, ((Integer) o).intValue());
            } else if (o instanceof Long) {
                i.set(key, ((Long) o).longValue());
            } else {
                i.set(key, o);
            }
        }
    }

    protected boolean evaluateExpression(String expr, WorkflowEntry entry, List registers) {
        try {
            Interpreter i = new Interpreter();
            Map variables = createVariableMap(entry, registers);
            setBshVariables(i, variables);
            // evaluate expression
            Object o = i.eval(expr);

            if (o == null) {
                return false;
            }

            return TextUtils.parseBoolean(o.toString());
        } catch (EvalError e) {
            log.error("Could not evaluate conditional expression", e);
            return false;
        } catch (PropertyException e) {
            log.error("Could not retrieve property value", e);
            return false;
        }
    }

    protected String translateVariables(String s, PropertySet ps) {
        while (true) {
            int x = s.indexOf("${");
            int y = s.indexOf("}", x);

            if (x != -1 && y != -1) {
                String var = s.substring(x + 2, y);
                String t = null;
                int schemaType = ps.getType(var);

                switch (schemaType) {
                    case PropertySet.BOOLEAN:
                        boolean b = ps.getBoolean(var);
                        if (b) {
                            t = "true";
                        } else {
                            t = "false";
                        }
                        break;
                    case PropertySet.DATA:
                        t = new String(ps.getData(var));
                        break;
                    case PropertySet.DATE:
                        t = ps.getDate(var).toString();
                        break;
                    case PropertySet.DOUBLE:
                        t = Double.toString(ps.getDouble(var));
                        break;
                    case PropertySet.INT:
                        t = Integer.toString(ps.getInt(var));
                        break;
                    case PropertySet.LONG:
                        t = Long.toString(ps.getLong(var));
                        break;
                    case PropertySet.OBJECT:
                        t = ps.getObject(var).toString();
                        break;
                    case PropertySet.PROPERTIES:
                        t = ps.getProperties(var).toString();
                        break;
                    case PropertySet.STRING:
                        t = ps.getString(var);
                        break;
                    case PropertySet.TEXT:
                        t = ps.getText(var);
                        break;
                    case PropertySet.XML:
                        t = XMLUtils.getElementText(ps.getXML(var).getDocumentElement());
                        break;
                }

                if (t != null) {
                    s = s.substring(0, x) + t + s.substring(y + 1);
                } else {
                    // the variable doesn't exist, so don't display anything
                    s = s.substring(0, x) + s.substring(y + 1);
                }
            } else {
                break;
            }
        }

        return s;
    }

    protected Persistence getPersistence() {
        return PersistenceFactory.getPersistence(context);
    }

    public PropertySet getPropertySet(long id) {
        try {
            Persistence p = getPersistence();
            WorkflowEntry entry = p.findWorkflowEntry(id);
            return entry.getPropertySet();
        } catch (Exception e) {
            throw new InternalWorkflowException(e);
        }
    }

    /**
     * Get an array of steps that have actions in the specified workflow
     * @param workflowName The name of the workflow
     * @return an array of step id's
     */
    public int[] getActionableSteps(String workflowName) {
        WorkflowDescriptor wf = getWorkflow(workflowName);

        List actionableSteps = new ArrayList();
        List steps = wf.getSteps();
        for (Iterator iterator = steps.iterator(); iterator.hasNext();) {
            StepDescriptor step = (StepDescriptor) iterator.next();
            List actions = step.getActions();
            for (Iterator iterator2 = actions.iterator(); iterator2.hasNext();) {
                ActionDescriptor action = (ActionDescriptor) iterator2.next();
                if (validRole(action.getRestriction().getGroups())) {
                    actionableSteps.add(new Integer(step.getId()));
                    break;
                }
            }

        }

        int[] x = new int[actionableSteps.size()];
        int i = 0;
        for (Iterator iterator = actionableSteps.iterator(); iterator.hasNext();) {
            Integer stepId = (Integer) iterator.next();
            x[i] = stepId.intValue();
            i++;
        }

        return x;
    }

  /**
   * Create a workflow entry
   * @param workflowName The name of the workflow to create
   * @return the workflow key id
   */
    public long createEntry(String workflowName) {
        try {
            Persistence p = getPersistence();
            WorkflowEntry entry = p.create(workflowName);
            p.persistChanges();
            return entry.getId();
        } catch (Exception e) {
            context.setRollbackOnly();
            log.error("Unable to create new workflow entry", e);
            throw new InternalWorkflowException(e);
        }
    }

    /**
     * Check if the specified workflow has been initialized
     * @param id The workflow id
     * @return true if the workflow has been initialized, false otherwise
     */
    public boolean isInitialized(long id) {
        Persistence p = getPersistence();
        WorkflowEntry entry = p.findWorkflowEntry(id);
        if (entry.getCurrentStep() != null) {
            return true;
        }

        return false;
    }

    /**
     * Check if the calling user has enough permissions to initialise the specified workflow
     * @param workflowName The name of the workflow to check
     * @param initialState The id of the initial state to check
     * @return true if the user can successfully call initialize, false otherwise
     */
    public boolean canInitialize(String workflowName, int initialState) {
        WorkflowDescriptor wf = getWorkflow(workflowName);
        List initialStates = wf.getInitialSteps();
        Iterator iter = initialStates.iterator();
        InitialStepDescriptor state = null;
        while(iter.hasNext())
        {
          InitialStepDescriptor descriptor = (InitialStepDescriptor)iter.next();
          if(descriptor.getId() == initialState) {
              state = descriptor;
          }
        }

        if(state == null) {
            throw new InternalWorkflowException("Invalid Initial State");
        }

        return validRole(state.getAllowableGroups());
    }

    public void initialize(long id, int initialState, Map inputs) throws InvalidRoleException, InvalidInputException {
        // don't allow a second initialization
        if (isInitialized(id) == true) {
            return;
        }

        Persistence p = getPersistence();
        WorkflowEntry entry = p.findWorkflowEntry(id);
        String workflowName = null;
        workflowName = entry.getWorkflowName();

        WorkflowDescriptor wf = getWorkflow(workflowName);
        List initialStates = wf.getInitialSteps();
        InitialStepDescriptor state = null;
        for (Iterator iterator = initialStates.iterator(); iterator.hasNext();) {
            InitialStepDescriptor is = (InitialStepDescriptor) iterator.next();
            if (is.getId() == initialState) {
                state = is;
            }
        }

        if (state == null) {
            throw new InternalWorkflowException("Invalid Initial State");
        }

        // check if the caller has a valid role to invoke this initialState
        if (!validRole(state.getAllowableGroups())) {
            throw new InvalidRoleException("Insufficient permissions to create new workflow");
        }

        // create WF entry
        PropertySet propertySet = null;
        try {
            propertySet = entry.getPropertySet();
        } catch (Exception e) {
            context.setRollbackOnly();
            throw new InternalWorkflowException(e);
        }

        // validate inputs (optional)
        verifyInputs(entry, state.getValidators(), inputs, wf.getRegisters());

        // preFunctions
        List preFunctions = state.getPreFunctions();
        for (Iterator iterator = preFunctions.iterator(); iterator.hasNext();) {
            FunctionDescriptor function = (FunctionDescriptor) iterator.next();
            executeFunction(function, entry, inputs, wf.getRegisters());
        }

        // initialize steps
        try {
            ResultDescriptor outcome = state.getUnconditionalResult();
            String owner = outcome.getOwner();
            if (outcome.getStatus() == com.opensymphony.workflow.loader.StatusValues.UNDERWAY && owner != null) {
                UserManager um = UserManager.getInstance();
                owner = translateVariables(owner, propertySet);

                // now make sure this user actually exists. If it doesn't, make the step be queued
                try {
                    User u = um.getUser(owner);
                    entry.initializeStep(outcome.getStep(), u);
                } catch (EntityNotFoundException e) {
                    entry.initializeStep(outcome.getStep());
                }
            } else {
                entry.initializeStep(outcome.getStep());
            }
        } catch (Exception e) {
             context.setRollbackOnly();
            throw new InternalWorkflowException(e);
        }

        // postFunctions
        List postFunctions = state.getPostFunctions();
        for (Iterator iterator = postFunctions.iterator(); iterator.hasNext();) {
            FunctionDescriptor function = (FunctionDescriptor) iterator.next();
            executeFunction(function, entry, inputs, wf.getRegisters());
        }

        p.persistChanges();
    }

    /**
     * perform an action on the specified workflow
     * @param id The workflow id to perform the action on
     * @param actionId The action id to perform (action id's are listed in the workflow descriptor)
     * @param inputs
     * @throws InvalidInputException
     */
    public void doAction(long id, int actionId, Map inputs) throws InvalidInputException {
        try {
            int[] availableActions = getAvailableActions(id);
            boolean validAction = false;
            for (int i = 0; i < availableActions.length; i++) {
                if (availableActions[i] == actionId) {
                    validAction = true;
                    break;
                }
            }
            if (!validAction) {
                return;
            }
        } catch (Exception e) {
            throw new InternalWorkflowException(e);
        }

        WorkflowDescriptor wf = null;
        WorkflowEntry entry = null;
        StepDescriptor s = null;
        Persistence p = getPersistence();
        entry = p.findWorkflowEntry(id);
        wf = getWorkflow(entry.getWorkflowName());
        com.opensymphony.workflow.persistence.Step step = entry.getCurrentStep();
        s = wf.getStep(step.getStepId());

        PropertySet propertySet = null;
        propertySet = entry.getPropertySet();

        ActionDescriptor action = s.getAction(actionId);
        //if (action == null || !validRole(action.getRoles())) {
        //    return;
        //}

        // validate inputs (optional)
        verifyInputs(entry, action.getValidators(), inputs, wf.getRegisters());

        // preFunctions
        List preFunctions = action.getPreFunctions();
        for (Iterator iterator = preFunctions.iterator(); iterator.hasNext();) {
            FunctionDescriptor function = (FunctionDescriptor) iterator.next();
            executeFunction(function, entry, inputs, wf.getRegisters());
        }

        // check each conditional result
        List conditionalResults = action.getConditionalResults();
        List extraPreFunctions = null;
        List extraPostFunctions = null;
        ResultDescriptor theResult = null;
        for (Iterator iterator = conditionalResults.iterator(); iterator.hasNext();) {
            ConditionalResultDescriptor conditionalResult = (ConditionalResultDescriptor) iterator.next();
            if (evaluateExpression(conditionalResult.getCondition(), entry, wf.getRegisters())) {
                theResult = conditionalResult;
                verifyInputs(entry, conditionalResult.getValidators(), inputs, wf.getRegisters());
                extraPreFunctions = conditionalResult.getPreFunctions();
                extraPostFunctions = conditionalResult.getPostFunctions();
                break;
            }
        }

        // use unconditional-result if a condition hasn't been met
        if (theResult == null) {
            theResult = action.getUnconditionalResult();
            verifyInputs(entry, theResult.getValidators(), inputs, wf.getRegisters());
            extraPreFunctions = theResult.getPreFunctions();
            extraPostFunctions = theResult.getPostFunctions();
        }

        // run any extra pre-functions that haven't been run already
        for (Iterator iterator = extraPreFunctions.iterator(); iterator.hasNext();) {
            FunctionDescriptor function = (FunctionDescriptor) iterator.next();
            executeFunction(function, entry, inputs, wf.getRegisters());
        }

        // go to next step
        try {
            log.debug("Outcome: stepId=" + theResult.getStep() + ", status=" + theResult.getStatus() + ", owner=" + theResult.getOwner());
            String owner = theResult.getOwner();
            if (theResult.getStatus() == com.opensymphony.workflow.loader.StatusValues.UNDERWAY && owner != null) {
                UserManager um = UserManager.getInstance();
                owner = translateVariables(owner, propertySet);

                // now make sure this user actually exists. If it doesn't, make the step be queued
                try {
                    User u = um.getUser(owner);
                    entry.newStep(theResult.getStep(), action.getId(), u);
                } catch (EntityNotFoundException e) {
                    entry.newStep(theResult.getStep(), action.getId());
                }
            } else if (theResult.getStatus() == com.opensymphony.workflow.loader.StatusValues.FINISHED) {
                UserManager um = UserManager.getInstance();
                owner = translateVariables(owner, propertySet);
                User u = um.getUser(owner);
                entry.finishStep(theResult.getStep(), actionId, u);
            } else {
                entry.newStep(theResult.getStep(), action.getId());
            }
        } catch (Exception e) {
            context.setRollbackOnly();
            throw new InternalWorkflowException(e);
        }

        // postFunctions (BOTH)
        for (Iterator iterator = extraPostFunctions.iterator(); iterator.hasNext();) {
            FunctionDescriptor function = (FunctionDescriptor) iterator.next();
            executeFunction(function, entry, inputs, wf.getRegisters());
        }
        List postFunctions = action.getPostFunctions();
        for (Iterator iterator = postFunctions.iterator(); iterator.hasNext();) {
            FunctionDescriptor function = (FunctionDescriptor) iterator.next();
            executeFunction(function, entry, inputs, wf.getRegisters());
        }

        p.persistChanges();
    }

    /**
     * Get an array of possible actions for the specified workflow
     * @param id the workflow to get actions for
     * @return an array of action id's. The collection of action is those that can be performed on the current step
     */
    public int[] getAvailableActions(long id) {
        WorkflowDescriptor wf = null;
        StepDescriptor s = null;
        Step step = null;
        Persistence p = getPersistence();
        WorkflowEntry entry = p.findWorkflowEntry(id);
        wf = getWorkflow(entry.getWorkflowName());
        step = entry.getCurrentStep();
        s = wf.getStep(step.getStepId());
        if (s == null) throw new InternalWorkflowException("Workflow is in non-existent step id " + step.getStepId());
        String owner = "";
        User u = step.getOwner();
        if (u != null) {
            owner = u.getName();
        }

        List l = new ArrayList();

        for (Iterator iterator = s.getActions().iterator(); iterator.hasNext();) {
            ActionDescriptor action = (ActionDescriptor) iterator.next();
            if (step.isStatus(action.getRestriction().getStatus()) &&
                    (validRole(action.getRestriction().getGroups()) ||
                    (action.getRestriction().getOwner() == RestrictionDescriptor.OWNER_ALLOW && context.getCaller().equals(owner))
                    )
            ) {
                String condition = action.getRestriction().getCondition();
                if (condition == null || (condition != null && evaluateExpression(condition, entry, wf.getRegisters()))) {
                    l.add(new Integer(action.getId()));
                }
            }
        }

        int[] actions = new int[l.size()];
        for (int i = 0; i < actions.length; i++) {
            actions[i] = ((Integer) l.get(i)).intValue();
        }

        return actions;
    }

    /**
     * Get a verbose string action name for a specified workflow action within the current step
     * @param id The workflow id to query
     * @param actionId The action id to get the name for
     * @return the step name
     */
    public String getActionName(long id, int actionId) {
        Persistence p = getPersistence();
        WorkflowEntry entry = p.findWorkflowEntry(id);
        int stepId = entry.getCurrentStep().getStepId();
        WorkflowDescriptor wf = getWorkflow(entry.getWorkflowName());
        StepDescriptor step = wf.getStep(stepId);
        ActionDescriptor action = step.getAction(actionId);
        return action.getName();
    }

    /**
     * Get a string name for the specified step
     * @param id The workflow id
     * @param stepId The step id
     * @return the name of the step
     */
    public String getStepName(long id, int stepId) {
        Persistence p = getPersistence();
        WorkflowEntry entry = p.findWorkflowEntry(id);
        WorkflowDescriptor wf = getWorkflow(entry.getWorkflowName());
        StepDescriptor step = wf.getStep(stepId);
        return step.getName();
    }

    /**
     * Get the workflow history
     * @param id the workflow id
     * @return A WorkflowHistory which contains a list of all steps the workflow has been in
     */
    public WorkflowHistory getHistory(long id) {
        try {
            Persistence p = getPersistence();
            WorkflowEntry entry = p.findWorkflowEntry(id);
            String workflowName = entry.getWorkflowName();
            WorkflowDescriptor wf = getWorkflow(workflowName);
            List steps = new ArrayList();
            Step step = entry.getCurrentStep();

            while (step != null) {
                int stepId = step.getStepId();
                StepDescriptor s = wf.getStep(stepId);
                String stepName = null;
                String actionName = null;
                int actionId = 0;
                if (s != null) {
                    stepName = s.getName();
                    if (step.getActionId() != 0) {
                        actionId = step.getActionId();
                        ActionDescriptor a = s.getAction(actionId);
                        if (a != null) {
                            actionName = a.getName();
                        }
                    }
                }
                User owner = step.getOwner();
                Date startDate = step.getStartDate();
                Date finishDate = step.getFinishDate();
                String status = "Unknown";
                if (step.isFinished()) {
                    status = "Finished";
                } else if (step.isQueued()) {
                    status = "Queued";
                } else if (step.isUnderway()) {
                    status = "Underway";
                }

                steps.add(new HistoryStep(stepName, stepId, actionName, actionId, owner, startDate, finishDate, status));

                step = step.getPrevious();
            }

            return new WorkflowHistory(id, workflowName, steps);
        } catch (Exception e) {
            throw new InternalWorkflowException(e);
        }
    }

    /**
     * Get the current status of the specified workflow
     * @param id The workflow ID
     * @return The step that the workflow is currently in
     */
    public HistoryStep getCurrentStatus(long id) {
        Persistence p = getPersistence();
        WorkflowEntry entry = p.findWorkflowEntry(id);
        String workflowName = entry.getWorkflowName();
        WorkflowDescriptor wf = getWorkflow(workflowName);
        Step step = entry.getCurrentStep();

        int stepId = step.getStepId();
        StepDescriptor s = wf.getStep(stepId);
        String stepName = null;
        String actionName = null;
        int actionId = 0;
        if (s != null) {
            stepName = s.getName();

            if (step.getActionId() != 0) {
                actionId = step.getActionId();
                ActionDescriptor a = s.getAction(actionId);
                if (a != null) {
                    actionName = a.getName();
                }
            }
        }
        User owner = step.getOwner();
        Date startDate = step.getStartDate();
        Date finishDate = step.getFinishDate();
        String status = "Unknown";
        if (step.isFinished()) {
            status = "Finished";
        } else if (step.isQueued()) {
            status = "Queued";
        } else if (step.isUnderway()) {
            status = "Underway";
        }

        return new HistoryStep(stepName, stepId, actionName, actionId, owner, startDate, finishDate, status);
    }

    /**
     * Get a collection of currently defined permissions for the specified workflow
     * @param id the workflow ID
     * @return A List of permissions specified currently (a permission is a string name)
     */
    public List getSecurityPermissions(long id) {
        Persistence p = getPersistence();
        WorkflowEntry entry = p.findWorkflowEntry(id);
        Step step = entry.getCurrentStep();
        String owner = "";
        User u = step.getOwner();
        if (u != null) {
            owner = u.getName();
        }
        int stepId = step.getStepId();

        WorkflowDescriptor wf = getWorkflow(entry.getWorkflowName());
        StepDescriptor xmlStep = wf.getStep(stepId);

        List securities = xmlStep.getPermissions();
        List s = new ArrayList();

        for (Iterator iterator = securities.iterator(); iterator.hasNext();) {
            PermissionDescriptor security = (PermissionDescriptor) iterator.next();
            if (step.isStatus(security.getRestriction().getStatus()) &&
                    (validRole(security.getRestriction().getGroups()) ||
                    (security.getRestriction().getOwner() == RestrictionDescriptor.OWNER_ALLOW && context.getCaller().equals(owner))
                    )
            ) {
                // to have the permission, the condition must be met or not specified
                String condition = security.getRestriction().getCondition();
                if (condition == null || (condition != null && evaluateExpression(condition, entry, wf.getRegisters()))) {
                    s.add(security.getName());
                }
            }

        }

        return s;
    }

    protected boolean validRole(List roles) {
        UserManager um = UserManager.getInstance();
        User user = null;
        try {
            user = um.getUser(context.getCaller());
        } catch (EntityNotFoundException e) {
            return false;
        }

        for (Iterator iterator = roles.iterator(); iterator.hasNext();) {
            String s = (String) iterator.next();
            if (user.inGroup(s)) {
                return true;
            }
        }
        return false;
    }

    protected void executeFunction(FunctionDescriptor function, WorkflowEntry entry, Map inputs, List registers) {
        if (function != null) {
            String type = function.getType();
            String location = function.getLocation();
            FunctionProvider provider;
            Map vars = createVariableMap(entry, registers);
            List registerNames = new ArrayList(registers.size());
            for (Iterator iterator = registers.iterator(); iterator.hasNext();) {
                RegisterDescriptor regDesc = (RegisterDescriptor) iterator.next();
                registerNames.add(regDesc.getVariableName());
            }

            if (type.equals("jndi")) {
                try {
                    provider = (FunctionProvider) EJBUtils.lookup(location);
                } catch (Exception e) {
                    throw new InternalWorkflowException("Unable to lookup FunctionProvider");
                }
            } else if (type.equals("class")) {
                try {
                    provider = (FunctionProvider) Class.forName(location).newInstance();
                } catch (Exception e) {
                    throw new InternalWorkflowException("Unable to lookup FunctionProvider");
                }
            } else {
                try {
                    Interpreter i = new Interpreter();
                    setBshVariables(i, vars);
                    i.set("inputs", inputs);
                    i.eval(function.getContents());
                    NameSpace nameSpace = i.getNameSpace();
                    String[] variables = nameSpace.getVariableNames();

                    for (int j = 0; j < variables.length; j++) {
                        if (isGoodVariable(variables[j], registerNames)) {
                            Object o = Primitive.unwrap(nameSpace.getVariable(variables[j], false));
                            if (o != null) {
                                entry.getPropertySet().setAsActualType(variables[j], o);
                            }
                        }
                    }
                } catch (EvalError e) {
                    log.error("Could not evaluate expression", e);
                }
                return;
            }

            PropertySet ps = null;
            ps = entry.getPropertySet();
            HashMap m = function.getArgs();
            for (Iterator iterator = m.entrySet().iterator(); iterator.hasNext();) {
                Map.Entry mapEntry = (Map.Entry) iterator.next();
                mapEntry.setValue(translateVariables((String) mapEntry.getValue(), ps));
            }

            provider.execute(inputs, m, vars);

            // save changes to the variable map
            Set entrySet = vars.entrySet();
            for (Iterator iterator = entrySet.iterator(); iterator.hasNext();) {
                Map.Entry me = (Map.Entry) iterator.next();
                Object key = me.getKey();
                String keyString = null;
                if (key instanceof String) {
                    keyString = (String) key;
                } else {
                    keyString = key.toString();
                }

                if (isGoodVariable(keyString, registerNames)) {
                    if (me.getValue() != null) {
                        entry.getPropertySet().setAsActualType(keyString, me.getValue());
                    }
                }
            }
        }
    }

    private boolean isGoodVariable(String varName, List registers) {
        // don't save variables 'entry', 'context', 'bsh', or a register name
        return (!"entry".equals(varName) && !"context".equals(varName) && !"bsh".equals(varName) && !"inputs".equals(varName) && !registers.contains(varName));
    }

    protected void verifyInputs(WorkflowEntry entry, List validators, Map inputs, List registers) throws InvalidInputException {
        Map variables = createVariableMap(entry, registers);
        for (Iterator iterator = validators.iterator(); iterator.hasNext();) {
            ValidatorDescriptor input = (ValidatorDescriptor) iterator.next();
            if (input != null) {
                String type = input.getType();
                String location = input.getLocation();
                com.opensymphony.workflow.Validator provider;
                if ("jndi".equals(type)) {
                    try {
                        provider = (com.opensymphony.workflow.Validator) EJBUtils.lookup(location);
                    } catch (Exception e) {
                        throw new InternalWorkflowException("Unable to lookup InputProvider!");
                    }
                } else if ("class".equals(type)) {
                    try {
                        provider = (com.opensymphony.workflow.Validator) Class.forName(location).newInstance();
                    } catch (Exception e) {
                        throw new InternalWorkflowException("Unable to lookup InputProvider!");
                    }
                } else {
                    // bsh validator
                    try {
                        Interpreter i = new Interpreter();
                        setBshVariables(i, variables);
                        i.set("inputs", inputs);
                        Object o = i.eval(input.getContents());
                        if (o != null) {
                            if (o instanceof InvalidInputException) {
                                throw (InvalidInputException) o;
                            }
                        }
                    } catch (EvalError e) {
                        log.error("Could not evaluate expression", e);
                        throw new InvalidInputException(e.getMessage());
                    }
                    return;
                }

                provider.validate(inputs, variables);
            }
        }
    }

    public void executeTriggerFunction(long id, int triggerId) {
        WorkflowDescriptor wf = null;
        WorkflowEntry entry = null;
        Persistence p = getPersistence();
        entry = p.findWorkflowEntry(id);
        wf = getWorkflow(entry.getWorkflowName());
        executeFunction(wf.getTriggerFunction(triggerId), entry, null, wf.getRegisters());
    }
}
