/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow;

import java.util.Map;

/**
 * Interface to be implemented by any class that are to be called from within a workflow as a function,
 * either as a pre-function or a post-function. The args nested elements within the function xml call
 * will be mapped to the properties parameter.
 *
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.9 $
 */
public interface FunctionProvider
{
    /**
     * Execute this function
     * @param inputs Workflow inputs
     * @param properties The properties for this function invocation. Properties are created from arg nested
     * elements within the xml, an arg element takes in a name attribute which is the properties key, and the CDATA
     * text contents of the element map to the property value.
     * @param variables The global, persistent variables that are associated with the current instance of the workflow.
     * Any change made to this map will be seen on the next function call in the workflow lifetime. There are two
     * special variable names: <b>entry</b> (object type: {@link com.opensymphony.workflow.persistence.WorkflowEntry WorkflowEntry}) and
     * <b>context</b> (object type: {@link com.opensymphony.workflow.WorkflowContext WorkflowContext}). These are always available in the variable map,
     * no matter what. Also, any variable set as a <b>register</b> (see: {@link com.opensymphony.workflow.Register Register}), will also
     * be available in the variable map, no matter what.
     */
    public void execute(Map inputs, Map properties, Map variables);

}
