/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow;

import java.rmi.RemoteException;

import com.opensymphony.workflow.persistence.WorkflowEntry;

/**
 * A remote register interface.
 * This interface is used when the register type is remote-ejb. The stateless session
 * bean would then implement this interface in order for a register to be successfully registered.
 *
 * @author <a href="hani@formicary.net">Hani Suleiman</a>
 * @version $Revision: 1.1 $
 */
public interface RegisterRemote
{
    /**
     * Register a variable within a particular workflow
     * @param context The current workflow context
     * @param entry The workflow entry. Note that this might be null, for example in a pre function
     * before the workflow has been initialised.
     * @return An object which is now exposed as a register to the rest of the workflow
     * @throws RemoteException
     */
    public Object registerVariable(WorkflowContext context, WorkflowEntry entry) throws RemoteException;
}
