/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow;

import com.opensymphony.module.propertyset.PropertySet;

import java.util.List;
import java.util.Map;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.9 $
 */
public interface Workflow {
    /**
     * Create a new workflow entry
     *
     * @param workflowName The name of the XML file for this workflow
     * @return The id of the new workflow
     */
    public long createEntry(String workflowName);

    public void initialize(long id, int initialState, Map inputs) throws InvalidRoleException, InvalidInputException;

    public boolean isInitialized(long id);

    public boolean canInitialize(String workflowName, int initialState);

    public int[] getAvailableActions(long id);

    public void doAction(long id, int actionId, Map inputs) throws InvalidInputException;

    public HistoryStep getCurrentStatus(long id);

    public WorkflowHistory getHistory(long id);

    public List getSecurityPermissions(long id);

    public int[] getActionableSteps(String workflowName);

    public String getActionName(long id, int actionId);

    public String getStepName(long id, int stepId);

    public PropertySet getPropertySet(long id);

    public void executeTriggerFunction(long id, int triggerId);

}
