/*
 * Created by IntelliJ IDEA.
 * User: plightbo
 * Date: Apr 29, 2002
 * Time: 12:16:00 PM
 */
package com.opensymphony.workflow.config;

import com.opensymphony.workflow.loader.WorkflowDescriptor;
import com.opensymphony.workflow.loader.AbstractWorkflowFactory;
import com.opensymphony.workflow.loader.URLWorkflowFactory;
import com.opensymphony.util.Logger;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import java.io.InputStream;
import java.util.Properties;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class ConfigLoader {
    public static String persistence;
    private static final Logger log = new Logger(ConfigLoader.class);
    private static AbstractWorkflowFactory factory = new URLWorkflowFactory();

    public static WorkflowDescriptor getWorkflow(String name) {
        WorkflowDescriptor workflow = factory.getWorkflow(name);
        if (workflow == null) {
            throw new RuntimeException("Unknown workflow name");
        }
        return workflow;
    }

    public static void load(InputStream is) {

        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware(true);
            DocumentBuilder db = null;

            try
            {
                db = dbf.newDocumentBuilder();
            }
            catch (ParserConfigurationException e)
            {
                log.fatal("Could not load config file", e);
            }

            Document doc = db.parse(is);

            Element root = (Element) doc.getElementsByTagName("osworkflow").item(0);
            Element p = (Element) root.getElementsByTagName("persistence").item(0);
            Element factoryElement = (Element)root.getElementsByTagName("factory").item(0);
            if(factoryElement!=null)
            {
              String clazz = null;
              try {
                clazz = factoryElement.getAttribute("class");
                factory = (AbstractWorkflowFactory)Class.forName(clazz).newInstance();
                Properties properties = new Properties();
                NodeList props = factoryElement.getElementsByTagName("property");
                for(int i=0;i<props.getLength();i++) {
                  Element e = (Element)props.item(i);
                  properties.setProperty(e.getAttribute("key"), e.getAttribute("value"));
                }
                factory.init(properties);
                factory.initDone();
              }
              catch(Exception ex)
              {
                throw new RuntimeException("Error creating workflow factory " + clazz + ": " + ex.getMessage());
              }
            }
            persistence = p.getAttribute("type");

        } catch (Exception e) {
            log.fatal(e);
            throw new RuntimeException("Error in workflow config: " + e.getMessage());
        }
    }
}
