/*
 * Created by IntelliJ IDEA.
 * User: plightbo
 * Date: Apr 29, 2002
 * Time: 10:33:46 PM
 */
package com.opensymphony.workflow.ejb;

import com.opensymphony.workflow.*;
import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.util.EJBUtils;
import com.opensymphony.util.Logger;

import javax.naming.NamingException;
import javax.ejb.CreateException;
import java.util.List;
import java.util.Map;
import java.rmi.RemoteException;

public class EJBWorkflow implements Workflow {

    private static final Logger log = new Logger(EJBWorkflow.class);
    private WorkflowRemote wf;

    public EJBWorkflow(String location) throws CreateException, RemoteException {
        WorkflowHome home = null;
        try {
            home = (WorkflowHome) EJBUtils.lookup(location, WorkflowHome.class);
        } catch (NamingException e) {
            try {
                home = (WorkflowHome) EJBUtils.lookup(WorkflowHome.COMP_NAME, WorkflowHome.class);
            } catch (NamingException e1) {
                try {
                    home = (WorkflowHome) EJBUtils.lookup(WorkflowHome.JNDI_NAME, WorkflowHome.class);
                } catch (NamingException e2) {
                    log.fatal(e);
                    throw new RuntimeException("Could not get a handle on the workflow EJB: " + e.getMessage());
                }
            }
        }

        wf = home.create();
    }

    public EJBWorkflow() throws CreateException, RemoteException {
        this(WorkflowHome.JNDI_NAME);
    }

    public long createEntry(String workflowName) {
        try {
            return wf.createEntry(workflowName);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public boolean canInitialize(String workflowName, int initialState) {
        try {
            return wf.canInitialize(workflowName, initialState);
        } catch(RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }

    }
    public void initialize(long id, int initialState, Map inputs) throws InvalidRoleException, InvalidInputException {
        try {
            wf.initialize(id, initialState, inputs);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public boolean isInitialized(long id) {
        try {
            return wf.isInitialized(id);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public int[] getAvailableActions(long id) {
        try {
            return wf.getAvailableActions(id);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public void doAction(long id, int actionId, Map inputs) throws InvalidInputException {
        try {
            wf.doAction(id, actionId, inputs);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public HistoryStep getCurrentStatus(long id) {
        try {
            return wf.getCurrentStatus(id);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public WorkflowHistory getHistory(long id) {
        try {
            return wf.getHistory(id);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public List getSecurityPermissions(long id) {
        try {
            return wf.getSecurityPermissions(id);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public int[] getActionableSteps(String workflowName) {
        try {
            return wf.getActionableSteps(workflowName);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public String getActionName(long id, int actionId) {
        try {
            return wf.getActionName(id, actionId);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public String getStepName(long id, int stepId) {
        try {
            return wf.getStepName(id, stepId);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public PropertySet getPropertySet(long id) {
        try {
            return wf.getPropertySet(id);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public void executeTriggerFunction(long id, int triggerId) {
        try {
            wf.executeTriggerFunction(id, triggerId);
        } catch (RemoteException e) {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }
}
