/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.ejb;

import com.opensymphony.workflow.HistoryStep;
import com.opensymphony.workflow.InvalidInputException;
import com.opensymphony.workflow.InvalidRoleException;
import com.opensymphony.workflow.WorkflowHistory;
import com.opensymphony.module.propertyset.PropertySet;

import javax.ejb.EJBObject;
import java.rmi.RemoteException;
import java.util.List;
import java.util.Map;

/**
 * Remote interface for Workflow.
 *
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.7 $
 */
public interface WorkflowRemote extends EJBObject
{
    public long createEntry(String workflowName) throws RemoteException;

    public void doAction(long id, int actionId, Map inputs) throws InvalidInputException, RemoteException;

    public String getActionName(long id, int actionId) throws RemoteException;

    public int[] getActionableSteps(String workflowName) throws RemoteException;

    public int[] getAvailableActions(long id) throws RemoteException;

    public HistoryStep getCurrentStatus(long id) throws RemoteException;

    public WorkflowHistory getHistory(long id) throws RemoteException;

    public List getSecurityPermissions(long id) throws RemoteException;

    public String getStepName(long id, int stepId) throws RemoteException;

    public PropertySet getPropertySet(long id) throws RemoteException;

    public boolean canInitialize(String workflowName, int initialState) throws RemoteException;

    public void initialize(long id, int initialState, Map inputs) throws InvalidRoleException, InvalidInputException, RemoteException;

    public boolean isInitialized(long id) throws RemoteException;

    public void executeTriggerFunction(long id, int triggerId) throws RemoteException;

}
