/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.loader;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.util.ArrayList;
import java.util.List;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.1 $
 */
public class BaseActionDescriptor
{
    private int id;
    private List validators;
    private List preFunctions;
    private List conditionalResults;
    private ResultDescriptor unconditionalResult;
    private List postFunctions;

    public BaseActionDescriptor(Element initialStep)
    {
        this.id = Integer.parseInt(initialStep.getAttribute("id"));
        this.validators = new ArrayList();
        this.preFunctions = new ArrayList();
        this.conditionalResults = new ArrayList();
        this.postFunctions = new ArrayList();

        // set up validators - OPTIONAL
        Element v = (Element) initialStep.getElementsByTagName("validators").item(0);
        if (v != null)
        {
            NodeList validators = v.getElementsByTagName("validator");
            for (int k = 0; k < validators.getLength(); k++)
            {
                Element validator = (Element) validators.item(k);
                this.validators.add(new ValidatorDescriptor(validator));
            }
        }

        // set up pre-functions - OPTIONAL
        Element pre = (Element) initialStep.getElementsByTagName("pre-functions").item(0);
        if (pre != null)
        {
            NodeList preFunctions = pre.getElementsByTagName("function");
            for (int k = 0; k < preFunctions.getLength(); k++)
            {
                Element preFunction = (Element) preFunctions.item(k);
                this.preFunctions.add(new FunctionDescriptor(preFunction));
            }
        }

        // set up results - REQUIRED
        Element resultsElememt = (Element) initialStep.getElementsByTagName("results").item(0);
        NodeList results = resultsElememt.getElementsByTagName("result");
        for (int k = 0; k < results.getLength(); k++)
        {
            Element result = (Element) results.item(k);
            this.conditionalResults.add(new ConditionalResultDescriptor(result));
        }
        Element unconditionalResult = (Element) resultsElememt.getElementsByTagName("unconditional-result").item(0);
        this.unconditionalResult = new ResultDescriptor(unconditionalResult);

        // set up post-functions - OPTIONAL
        Element post = (Element) initialStep.getElementsByTagName("post-functions").item(0);
        if (post != null)
        {
            NodeList postFunctions = post.getElementsByTagName("function");
            for (int k = 0; k < postFunctions.getLength(); k++)
            {
                Element postFunction = (Element) postFunctions.item(k);
                this.postFunctions.add(new FunctionDescriptor(postFunction));
            }
        }

    }

    public int getId()
    {
        return id;
    }

    public List getValidators()
    {
        return validators;
    }

    public List getPreFunctions()
    {
        return preFunctions;
    }

    public List getConditionalResults()
    {
        return conditionalResults;
    }

    public ResultDescriptor getUnconditionalResult()
    {
        return unconditionalResult;
    }

    public List getPostFunctions()
    {
        return postFunctions;
    }
}
