/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.loader;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.util.ArrayList;
import java.util.List;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.3 $
 */
public class RestrictionDescriptor implements StatusValues
{
    public static final int OWNER_ALLOW = 0;
    public static final int OWNER_DENY = 1;
    public static final int OWNER_USE_GROUPS = 2;

    private int status;
    private int owner;
    private List groups;
    private String condition;

    public RestrictionDescriptor(Element restriction)
    {
        String s = restriction.getAttribute("status");
        if ("Finished".equals(s))
        {
            status = FINISHED;
        }
        else if ("Underway".equals(s))
        {
            status = UNDERWAY;
        }
        else
        {
            status = QUEUED;
        }

        s = restriction.getAttribute("owner");
        if ("allow".equals(s))
        {
            owner = OWNER_ALLOW;
        }
        else if ("deny".equals(s))
        {
            owner = OWNER_DENY;
        }
        else
        {
            owner = OWNER_USE_GROUPS;
        }

        this.groups = new ArrayList();
        this.condition = restriction.getAttribute("condition");
        if ("".equals(this.condition.trim())) {
            this.condition = null;
        }

        // set up groups -- OPTIONAL
        NodeList groups = restriction.getElementsByTagName("group");
        for (int k = 0; k < groups.getLength(); k++)
        {
            Element group = (Element) groups.item(k);
            this.groups.add(group.getChildNodes().item(0).getNodeValue());
        }
    }

    public int getStatus()
    {
        return status;
    }

    public int getOwner()
    {
        return owner;
    }

    public List getGroups()
    {
        return groups;
    }

    public String getCondition() {
        return condition;
    }
}
