/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.loader;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.util.ArrayList;
import java.util.List;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.2 $
 */
public class ResultDescriptor implements StatusValues
{
    private int status;
    private int step;
    private String owner;
    private List validators;
    private List preFunctions;
    private List postFunctions;

    public ResultDescriptor(Element result)
    {
        String s = result.getAttribute("status");
        if ("Finished".equals(s))
        {
            status = FINISHED;
        }
        else if ("Underway".equals(s))
        {
            status = UNDERWAY;
        }
        else
        {
            status = QUEUED;
        }

        step = Integer.parseInt(result.getAttribute("step"));
        owner = result.getAttribute("owner");
        this.validators = new ArrayList();
        this.preFunctions = new ArrayList();
        this.postFunctions = new ArrayList();

        // set up validators -- OPTIONAL
        Element v = (Element) result.getElementsByTagName("validators").item(0);
        if (v != null)
        {
            NodeList validators = v.getElementsByTagName("validator");
            for (int k = 0; k < validators.getLength(); k++)
            {
                Element validator = (Element) validators.item(k);
                this.validators.add(new ValidatorDescriptor(validator));
            }
        }

        // set up pre-functions -- OPTIONAL
        Element pre = (Element) result.getElementsByTagName("pre-functions").item(0);
        if (pre != null)
        {
            NodeList preFunctions = pre.getElementsByTagName("function");
            for (int k = 0; k < preFunctions.getLength(); k++)
            {
                Element preFunction = (Element) preFunctions.item(k);
                this.preFunctions.add(new FunctionDescriptor(preFunction));
            }
        }

        // set up post-functions - OPTIONAL
        Element post = (Element) result.getElementsByTagName("post-functions").item(0);
        if (post != null)
        {
            NodeList postFunctions = post.getElementsByTagName("function");
            for (int k = 0; k < postFunctions.getLength(); k++)
            {
                Element postFunction = (Element) postFunctions.item(k);
                this.postFunctions.add(new FunctionDescriptor(postFunction));
            }
        }
    }

    public int getStatus()
    {
        return status;
    }

    public int getStep()
    {
        return step;
    }

    public String getOwner()
    {
        return owner;
    }

    public List getValidators()
    {
        return validators;
    }

    public List getPreFunctions() {
        return preFunctions;
    }

    public List getPostFunctions()
    {
        return postFunctions;
    }
}
