/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.loader;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.2 $
 */
public class StepDescriptor
{
    private int id;
    private String name;
    private List permissions;
    private List actions;

    public StepDescriptor(Element step)
    {
        id = Integer.parseInt(step.getAttribute("id"));
        name = step.getAttribute("name");

        this.permissions = new ArrayList();
        this.actions = new ArrayList();

        // set up permissions - OPTIONAL
        Element p = (Element) step.getElementsByTagName("external-permissions").item(0);
        if (p != null)
        {
            NodeList permissions = p.getElementsByTagName("permission");
            for (int i = 0; i < permissions.getLength(); i++)
            {
                Element permission = (Element) permissions.item(i);
                this.permissions.add(new PermissionDescriptor(permission));
            }
        }

        // set up actions - OPTIONAL
        Element a = (Element) step.getElementsByTagName("actions").item(0);
        if (a != null)
        {
            NodeList actions = a.getElementsByTagName("action");
            for (int i = 0; i < actions.getLength(); i++)
            {
                Element action = (Element) actions.item(i);
                this.actions.add(new ActionDescriptor(action));
            }
        }
    }

    public int getId()
    {
        return id;
    }

    public String getName()
    {
        return name;
    }

    /**
     * Get a List of {@link PermissionDescriptor}s for this step
     * @return
     */
    public List getPermissions()
    {
        return permissions;
    }

    /**
     * Get a List of {@link ActionDescriptor}s for this step
     */
    public List getActions()
    {
        return actions;
    }

    public ActionDescriptor getAction(int id)
    {
        for (Iterator iterator = actions.iterator(); iterator.hasNext();)
        {
            ActionDescriptor action = (ActionDescriptor) iterator.next();
            if (action.getId() == id)
            {
                return action;
            }
        }

        return null;
    }
}
