package com.opensymphony.workflow.loader;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import com.opensymphony.util.Logger;

/**
 * @author Hani Suleiman
 * Date: May 10, 2002
 * Time: 11:59:47 AM
 */
public class URLWorkflowFactory extends AbstractWorkflowFactory
{
  private static final Logger log = new Logger(URLWorkflowFactory.class);

  private Map cache = new HashMap();

  public WorkflowDescriptor getWorkflow(String name)
  {
    boolean useCache = getProperties().getProperty("cache", "false").equals("true");
    if(useCache)
    {
      WorkflowDescriptor descriptor = (WorkflowDescriptor)cache.get(name);
      if(descriptor!=null) return descriptor;
    }
    try
    {
      URL url = new URL(name);
      WorkflowDescriptor descriptor = WorkflowLoader.load(url);
      if(useCache)
        cache.put(name, descriptor);
      return descriptor;
    }
    catch(Exception e)
    {
      log.fatal("Unable to find workflow " + name, e);
    }
    return null;
  }
}
