package com.opensymphony.workflow.loader;

import java.util.Map;
import java.util.HashMap;
import java.net.URL;
import java.io.File;
import java.io.InputStream;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import com.opensymphony.util.Logger;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/**
 * @author Hani Suleiman
 * Date: May 10, 2002
 * Time: 11:30:41 AM
 */
public class XMLWorkflowFactory extends AbstractWorkflowFactory {

  private static final Logger log = new Logger(XMLWorkflowFactory.class);

  private Map workflows;

  class WorkflowConfig {
      public WorkflowConfig(String type, String location) {
          this.type = type;
          this.location = location;
      }

      String type, location;
  }

  public WorkflowDescriptor getWorkflow(String name) {
    WorkflowConfig c = (WorkflowConfig) workflows.get(name);
    if (c == null) {
        throw new RuntimeException("Unknown workflow name");
    }

    try {
        if (c.type.equalsIgnoreCase("URL")) {
            return WorkflowLoader.load(new URL(c.location));
        } else if (c.type.equalsIgnoreCase("file")) {
            return WorkflowLoader.load(new File(c.location));
        } else {
            return WorkflowLoader.load(c.location);
        }
    } catch (Exception e) {
        log.fatal("Error creating workflow descriptor", e);
        throw new RuntimeException("Error in workflow descriptor: " + e.getMessage());
    }
  }

  public void initDone() throws Exception
  {
    ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
    InputStream is = null;
    String name = getProperties().getProperty("resource", "workflows.xml");
    try {
        is = classLoader.getResourceAsStream(name);
    } catch (Exception e) {
    }

    if (is == null) {
        try {
            is = classLoader.getResourceAsStream("/" + name);
        } catch (Exception e) {
        }
    }

    if (is == null) {
        try {
            is = classLoader.getResourceAsStream("META-INF/" + name);
        } catch (Exception e) {
        }
    }

    if (is == null) {
        try {
            is = classLoader.getResourceAsStream("/META-INF/" + name);
        } catch (Exception e) {
        }
    }

    try {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        DocumentBuilder db = null;

        try
        {
            db = dbf.newDocumentBuilder();
        }
        catch (ParserConfigurationException e)
        {
            log.fatal("Error creating document builder", e);
        }

        Document doc = db.parse(is);

        Element root = (Element) doc.getElementsByTagName("workflows").item(0);
        workflows = new HashMap();
        NodeList list = root.getElementsByTagName("workflow");
        for (int i = 0; i < list.getLength(); i++) {
            Element e = (Element) list.item(i);
            workflows.put(e.getAttribute("name"), new WorkflowConfig(e.getAttribute("type"), e.getAttribute("location")));
        }
    } catch (Exception e) {
        log.fatal("Error reading xml workflow", e);
        throw new RuntimeException("Error in workflow config: " + e.getMessage());
    }
  }
}
