/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.persistence;

import com.opensymphony.module.user.User;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.3 $
 */
public interface Persistence
{
    public WorkflowEntry findWorkflowEntry(long pk);

    public WorkflowEntry create(java.lang.String workflowName);

    /**
     * Create new step with no previous step.
     * This new step is marked as QUEUED.
     */
    public Step create(long entryId, int stepId);

    /**
     * Create a new step with no previous step.
     * This new step is marked as UNDERWAY.
     */
    public Step create(long entryId, int stepId, User owner);

    /**
     * Create a new step with an REQUIRED previous step that MUST be marked finished.
     * This new step is marked as queued.
     */
    public Step create(Step prev, int stepId, int actionId);

    /**
     * Create a new step with a REQUIRED previous step that MUST be marked finished.
     * This new step is marked as UNDERWAY
     */
    public Step create(Step prev, int stepId, int actionId, User owner);

    public Step create(Step prev, int actionId);

    public Step findStep(long pk);

    public void persistChanges();

}
