/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.persistence.ejb;

import com.opensymphony.module.user.User;
import com.opensymphony.util.EJBUtils;
import com.opensymphony.util.Logger;
import com.opensymphony.workflow.persistence.Persistence;
import com.opensymphony.workflow.persistence.Step;
import com.opensymphony.workflow.persistence.WorkflowEntry;
import com.opensymphony.workflow.persistence.ejb.entry.WorkflowEntryHome;
import com.opensymphony.workflow.persistence.ejb.step.StepHome;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.5 $
 */
public class EJBPersistence implements Persistence
{
    private static final Logger log = new Logger(EJBPersistence.class);

    public WorkflowEntry findWorkflowEntry(long pk)
    {
        try
        {
            WorkflowEntryHome home = (WorkflowEntryHome) EJBUtils.lookup(WorkflowEntryHome.COMP_NAME, WorkflowEntryHome.class);
            return new EJBWorkflowEntry(home.findByPrimaryKey(new Long(pk)));
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public WorkflowEntry create(String workflowName)
    {
        try
        {
            WorkflowEntryHome home = (WorkflowEntryHome) EJBUtils.lookup(WorkflowEntryHome.COMP_NAME, WorkflowEntryHome.class);
            return new EJBWorkflowEntry(home.create(workflowName));
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Step create(long entryId, int stepId)
    {
        try
        {
            StepHome home = (StepHome) EJBUtils.lookup(StepHome.COMP_NAME, StepHome.class);
            return new EJBStep(home.create(new Long(entryId), stepId));
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Step create(long entryId, int stepId, User owner)
    {
        try
        {
            StepHome home = (StepHome) EJBUtils.lookup(StepHome.COMP_NAME, StepHome.class);
            return new EJBStep(home.create(new Long(entryId), stepId, owner));
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Step create(Step prev, int stepId, int actionId)
    {
        try
        {
            StepHome home = (StepHome) EJBUtils.lookup(StepHome.COMP_NAME, StepHome.class);
            return new EJBStep(home.create(home.findByPrimaryKey(new Long(prev.getId())), stepId, actionId));
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Step create(Step prev, int stepId, int actionId, User owner)
    {
        try
        {
            StepHome home = (StepHome) EJBUtils.lookup(StepHome.COMP_NAME, StepHome.class);
            return new EJBStep(home.create(home.findByPrimaryKey(new Long(prev.getId())), stepId, actionId, owner));
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Step create(Step prev, int actionId)
    {
        try
        {
            StepHome home = (StepHome) EJBUtils.lookup(StepHome.COMP_NAME, StepHome.class);
            return new EJBStep(home.create(home.findByPrimaryKey(new Long(prev.getId())), actionId));
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Step findStep(long pk)
    {
        try
        {
            StepHome home = (StepHome) EJBUtils.lookup(StepHome.COMP_NAME, StepHome.class);
            return new EJBStep(home.findByPrimaryKey(new Long(pk)));
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public void persistChanges()
    {
        // nothing required, container does this
    }
}
