/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.persistence.ejb;

import com.opensymphony.module.user.User;
import com.opensymphony.workflow.persistence.Step;
import com.opensymphony.workflow.persistence.ejb.step.StepRemote;
import com.opensymphony.util.Logger;

import java.util.Date;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.6 $
 */
public class EJBStep implements Step
{
    private static final Logger log = new Logger(EJBStep.class);

    private StepRemote step;

    public EJBStep(StepRemote step)
    {
        this.step = step;
    }

    public int getActionId()
    {
        try
        {
            Long l = step.getActionId();
            if (l == null)
            {
                return 0;
            }
            else
            {
                return step.getActionId().intValue();
            }
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public long getEntryId()
    {
        try
        {
            return step.getEntryId().longValue();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Date getFinishDate()
    {
        try
        {
            return step.getFinishDate();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public long getId()
    {
        try
        {
            return step.getId().longValue();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public User getOwner()
    {
        try
        {
            return step.getOwner();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Step getPrevious()
    {
        try
        {
            StepRemote s = step.getPrevious();
            if (s == null)
            {
                return null;
            }
            else
            {
                return new EJBStep(step.getPrevious());
            }
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Date getStartDate()
    {
        try
        {
            return step.getStartDate();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public int getStepId()
    {
        try
        {
            return step.getStepId().intValue();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public boolean isFinished()
    {
        try
        {
            return step.isFinished();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public boolean isQueued()
    {
        try
        {
            return step.isQueued();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public boolean isStatus(int status)
    {
        try
        {
            return step.isStatus(status);
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public boolean isUnderway()
    {
        try
        {
            return step.isUnderway();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public void markFinished()
    {
        try
        {
            step.markFinished();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public void markUnderway(User owner)
    {
        try
        {
            step.markUnderway(owner);
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public void setActionId(int id)
    {
        try
        {
            step.setActionId(new Long(id));
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }
}
