/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.persistence.ejb;

import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.module.user.User;
import com.opensymphony.workflow.persistence.Step;
import com.opensymphony.workflow.persistence.WorkflowEntry;
import com.opensymphony.workflow.persistence.ejb.entry.WorkflowEntryRemote;
import com.opensymphony.workflow.persistence.ejb.step.StepRemote;
import com.opensymphony.util.Logger;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.7 $
 */
public class EJBWorkflowEntry implements WorkflowEntry
{
    private static final Logger log = new Logger(EJBWorkflowEntry.class);

    private WorkflowEntryRemote entry;

    public EJBWorkflowEntry(WorkflowEntryRemote entry)
    {
        this.entry = entry;
    }

    public void finishStep(int stepId, int actionId, User owner)
    {
        try
        {
            entry.finishStep(stepId, actionId, owner);
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Step getCurrentStep()
    {
        try
        {
            StepRemote s = entry.getCurrentStep();
            if (s == null)
            {
                return null;
            }
            else
            {
                return new EJBStep(entry.getCurrentStep());
            }
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public long getId()
    {
        try
        {
            return entry.getId().longValue();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public PropertySet getPropertySet()
    {
        try
        {
            return entry.getPropertySet();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public String getWorkflowName()
    {
        try
        {
            return entry.getWorkflowName();
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public void initializeStep(int stepId)
    {
        try
        {
            entry.initializeStep(stepId);
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public void initializeStep(int stepId, User owner)
    {
        try
        {
            entry.initializeStep(stepId, owner);
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public void newStep(int stepId, int actionId)
    {
        try
        {
            entry.newStep(stepId, actionId);
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public void newStep(int stepId, int actionId, User owner)
    {
        try
        {
            entry.newStep(stepId, actionId, owner);
        }
        catch (Exception e)
        {
            log.error(e);
            throw new RuntimeException(e.getMessage());
        }
    }
}
