/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.persistence.ejb.entry;

import com.opensymphony.ejb.EntityAdapter;
import com.opensymphony.module.propertyset.EJBPropertySet;
import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.module.propertyset.ejb.PropertyStoreHome;
import com.opensymphony.module.user.User;
import com.opensymphony.module.user.UserManager;
import com.opensymphony.util.EJBUtils;
import com.opensymphony.workflow.persistence.ejb.step.StepRemote;
import com.opensymphony.workflow.persistence.ejb.step.StepHome;

import javax.ejb.CreateException;
import javax.ejb.EJBException;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.3 $
 */
public abstract class WorkflowEntryEJB extends EntityAdapter
{
	public abstract Long getId();

	public abstract void setId(Long id);

	public abstract Long getCurrentStepId();

	public abstract void setCurrentStepId(Long stepId);

    public abstract String getWorkflowName();

	public abstract void setWorkflowName(String stepId);

	public StepRemote getCurrentStep()
	{
		try
		{
			StepHome home = (StepHome) EJBUtils.lookup("ejb/Step", StepHome.class);
            Long pk = getCurrentStepId();
            if (pk != null && pk.longValue() != 0) {
			    return home.findByPrimaryKey(pk);
            }
		}
		catch (Exception e)
		{
		}

        return null;
	}

	public void initializeStep(long stepId)
	{
		try
		{
			StepHome home = (StepHome) EJBUtils.lookup("ejb/Step", StepHome.class);

			setCurrentStepId(home.create(getId(), stepId).getId());
		}
		catch (Exception e)
		{
			throw new EJBException(e);
		}
	}

	public void initializeStep(long stepId, User owner)
	{
		try
		{
			StepHome home = (StepHome) EJBUtils.lookup("ejb/Step", StepHome.class);

			setCurrentStepId(home.create(getId(), stepId, owner).getId());
		}
		catch (Exception e)
		{
			throw new EJBException(e);
		}
	}

	public void newStep(long stepId, long actionId)
	{
		try
		{
			StepHome home = (StepHome) EJBUtils.lookup("ejb/Step", StepHome.class);
			StepRemote currentStep = getCurrentStep();

			if (currentStep.isQueued())
			{
				// since it was never even started, make it underway as the current caller
				User caller = UserManager.getInstance().getUser(context.getCallerPrincipal().getName());
				currentStep.markUnderway(caller);
			}

			if (currentStep.isUnderway())
			{
				currentStep.markFinished();
			}

			setCurrentStepId(home.create(currentStep, stepId, actionId).getId());
		}
		catch (Exception e)
		{
		}
	}

	public void newStep(long stepId, long actionId, User owner)
	{
		try
		{
			StepHome home = (StepHome) EJBUtils.lookup("ejb/Step", StepHome.class);
			StepRemote currentStep = getCurrentStep();

			if (currentStep.isQueued())
			{
				// since it was never even start, make it underway as the current caller
				User caller = UserManager.getInstance().getUser(context.getCallerPrincipal().getName());
				currentStep.markUnderway(caller);
			}

			if (currentStep.isUnderway())
			{
				currentStep.markFinished();
			}

			setCurrentStepId(home.create(currentStep, stepId, actionId, owner).getId());
		}
		catch (Exception e)
		{
		}
	}

	public void finishStep(long stepId, long actionId, User owner)
	{
		try
		{
			StepHome home = (StepHome) EJBUtils.lookup("ejb/Step", StepHome.class);
			StepRemote currentStep = getCurrentStep();

			if (currentStep.isQueued())
			{
				User caller = UserManager.getInstance().getUser(context.getCallerPrincipal().getName());
				currentStep.markUnderway(caller);
			}

			if (currentStep.isUnderway())
			{
				currentStep.markFinished();
			}

			setCurrentStepId(home.create(currentStep, stepId, actionId, owner).getId());
			currentStep = getCurrentStep();
			currentStep.markFinished();
		}
		catch (Exception e)
		{
		}
	}

	public PropertySet getPropertySet()
	{
		try
		{
			PropertyStoreHome home = (PropertyStoreHome) EJBUtils.lookup("ejb/propertyStore", PropertyStoreHome.class);
			return new EJBPropertySet(home, "WorkflowEntry", getId().longValue());
		}
		catch (Exception e)
		{
			return null;
		}
	}

	public Long ejbCreate(String workflowName) throws CreateException
	{
		try
		{
			Long id = new Long(nextLong());
			setId(id);
			setWorkflowName(workflowName);
			return id;
		}
		catch (Exception e)
		{
			throw new CreateException(e.getMessage());
		}
	}

	public void ejbPostCreate(String workflowName) throws CreateException
	{
	}

}
