/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.persistence.ofbiz;

import com.opensymphony.module.user.User;
import com.opensymphony.module.user.UserManager;
import com.opensymphony.module.user.EntityNotFoundException;
import com.opensymphony.workflow.persistence.Step;
import com.opensymphony.workflow.loader.StatusValues;
import com.opensymphony.util.DataUtil;
import com.opensymphony.util.Logger;

import java.util.Date;
import java.sql.Timestamp;

import org.ofbiz.core.entity.GenericValue;
import org.ofbiz.core.entity.GenericEntityException;
import org.ofbiz.core.util.UtilDateTime;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.7 $
 */
public class OfbizStep implements Step, StatusValues {
    private static final Logger log = new Logger(OfbizStep.class);

    protected GenericValue gv;

    public OfbizStep(GenericValue gv) {
        this.gv = gv;
    }

    public void setPreviousId(long id) {
        gv.set("prevId", new Long(id));
    }

    public void setEntryId(long id) {
        gv.set("entryId", new Long(id));
    }

    public int getActionId() {
        return DataUtil.getInt(gv.getInteger("actionId"));
    }

    public long getEntryId() {
        return gv.getLong("entryId").longValue();
    }

    public Date getFinishDate() {
        return (Timestamp) gv.getTimestamp("finishDate");
    }

    public long getId() {
        return gv.getLong("id").longValue();
    }

    public User getOwner() {
        String owner = gv.getString("owner");
        if(owner==null) return null;
          try {
              UserManager um = UserManager.getInstance();
              return um.getUser(owner);
          } catch (EntityNotFoundException e) {
              return null;
          }
    }

    public Step getPrevious() {
        try {
            GenericValue prev = gv.getRelatedOne("prevStepstep");
            if (prev == null)
                return null;
            return new OfbizStep(prev);
        } catch (GenericEntityException e) {
            log.error("Unable to get previous step for step #" + getId(), e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Date getStartDate() {
        return (Timestamp) gv.getTimestamp("startDate");
    }

    public int getStepId() {
        return DataUtil.getInt(gv.getInteger("stepId"));
    }

    public boolean isFinished() {
        return (gv.getInteger("status").intValue() == FINISHED);
    }

    public boolean isQueued() {
        return (gv.getInteger("status").intValue() == QUEUED);
    }

    public boolean isStatus(int status) {
        return (gv.getInteger("status").intValue() == status);
    }

    public boolean isUnderway() {
        return (gv.getInteger("status").intValue() == UNDERWAY);
    }

    public void markFinished() {
        if (isUnderway()) {
            gv.set("finishDate", UtilDateTime.nowTimestamp());
            gv.set("status", new Integer(FINISHED));
        }
    }

    public void markUnderway(User owner) {
        gv.set("startDate", UtilDateTime.nowTimestamp());
        gv.set("owner", owner.getName());
        gv.set("status", new Integer(UNDERWAY));
    }

    public void setActionId(int id) {
        gv.set("actionId", new Integer(id));
    }
}
