/*
 * Created by IntelliJ IDEA.
 * User: plightbo
 * Date: May 23, 2002
 * Time: 2:33:59 AM
 */
package com.opensymphony.workflow.util;

import com.opensymphony.workflow.FunctionProvider;
import com.opensymphony.workflow.persistence.WorkflowEntry;
import com.opensymphony.util.Logger;

import java.util.Map;

import org.quartz.Scheduler;
import org.quartz.impl.StdSchedulerFactory;

/**
 * Unschedules a job that was scheduled previously. Accepts the following arguments:
 *
 * <ul>
 *  <li>triggerName - the name of the trigger previously scheduled</li>
 *  <li>groupName - the name of the group previously scheduled</li>
 * </ul>
 *
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.4 $
 */
public class UnscheduleJob implements FunctionProvider {
    private static final Logger log = new Logger(UnscheduleJob.class);

    public void execute(Map inputs, Map properties, Map variables) {
        try {
            WorkflowEntry entry = (WorkflowEntry) variables.get("entry");

            log.info("Starting to unschedule job for WF #" + entry.getId());

            Scheduler s = new StdSchedulerFactory().getScheduler();
            String triggerName = (String) properties.get("triggerName");
            String groupName = (String) properties.get("groupName");
            triggerName = triggerName + ":"+ entry.getId();
            groupName = groupName + ":" + entry.getId();
            s.unscheduleJob(triggerName, groupName);

            log.info("Job unscheduled");
        } catch (Exception e) {
            log.error("Could not unschedule job", e);
        }
    }
}
